#!/usr/bin/env perl
#
#  Copyright 1996-2009 Jim Pirzyk / pirzyk.org
#  All rights reserved.
#
#  install.pl - Install script to install the security package
#  and generate a sane default security.cf config file
#

use Getopt::Std;
use Sys::Filesystem;
use warnings;
use strict;

our $opts = 'Im:M:P:R:hy';
our %opts;
our $config = {
	checksumfiles	=> [ '/etc/passwd', '/etc/group', '/etc/shadow', '/etc/sudoers', '/etc/ssh/sshd_config', '/etc/ssh/ssh_config' ],
	ignoredevdirs	=> [ '/dev', '/compat/linux/dev', '/dev/fd', '/selinux' ],
	ignorefs		=> [ 'nfs', 'autofs', 'iso9660', 'hsfs', 'cd9660', 'devfs', 'procfs' ],
	intruderfiles	=> [ '~root/.ncftp', '/usr/lib/libsn.a', '/dev/reset', '/dev/pmcf1', '/dev/pmcf2', '/dev/pmcf3', '/dev/pmcf4', '/tmp/.w0rm', '/tmp/.X11x', '/tmp/.w0rm0r', '/tmp/w0rmishere' ],
	mailfiles		=> [ '.forward', '.procmailrc' ],
};

sub usage {
	print "Usage: $0 [-hxI] [-P PREFIX] [-m mailhost] [-M MODULE DIR]\n";
	print "\t-h\tThis help screen\n";
	print "\t-m mailhost\tSMTP mail server (defaults to localhost)\n";
	print "\t-M Module DIR\tWhere the Perl modules are installed\n";
	print "\t-P PREFIX DIR\tWhere the program starts\n";
	print "\t-R ROOT DIR\tDirectory where / will be relocated to (for package building)\n";
	print "\t-I\tNon Interactive install\n";
	print "\t-y\tDefault answer is yes (instead of no)\n";
	exit 0;
}

sub unique {
	my (%a);

	map { $a{$_} = 1 } @_;

	return keys %a;
}

sub ask_question {
	my ($question, $default) = @_;
	my ($result);

	if ( ! exists $opts{'I'} ) {
		printf STDERR "$question [$default] ";
		$result = <STDIN>;
		chomp $result;
	}

	$result = $default if ( ! defined $result || ! length $result );

	return $result;
}

sub generate_config_file {
	my ($contents, @array);

	# Look through all the mount points for special type of filesystems to exclude from our find call.
     my $fs = new Sys::Filesystem;
	foreach my $fs ( $fs->special_filesystems() ) {
     	push @array, $fs if ( -d $fs );
	}
	$contents .= 'ignoredir ' . join (' ', @array) . "\n"
		if ( scalar @array );

	$contents .= 'ignorefs ' . join (' ', @{$config->{'ignorefs'}}) . "\n"
		if ( exists $config->{'ignorefs'} && scalar @{$config->{'ignorefs'}} );

	# now look for directories that should containt block and character special devices.
	@array = ();
	foreach my $d (@{$config->{'ignoredevdirs'}}) {
		push @array, $d
			if ( -d $d );
	}
	# TODO: Need to see if anonftp or some other chroot environment is on the
	# system to add that /dev location.
	$contents .= 'ignoredevdir ' . join (' ', @array) . "\n"
		if ( scalar @array );

	# Go through all the system accounts and record some interesting info
	use User::pwent;
	my (@nopass, @uid0, @sharedhome, %homedirs, $res);
	while (my $pw = getpwent) {
		
		# If we found an account w/o a password, ask if we should put
		# it in the list of accounts to allow to have no password
		# example of this is the 'sync' account on SunOS 4.x
		if (! defined $pw->passwd || ! length ($pw->passwd)) {
			$res = ask_question ('Should this account have no password (' . $pw->name . ')?', ( exists $opts{'y'} ? 'yes': 'no'));
			push @nopass, $pw->name
				if ( $res eq 'yes');
		}

		# Check out alternate root accounts.
		if ( $pw->uid == 0 && $pw->name ne 'root' ) {
			$res = ask_question ('Should this uid 0 account exist (' . $pw->name . ')?', ( exists $opts{'y'} ? 'yes': 'no'));
			push @uid0, $pw->name
				if ( $res eq 'yes');
		}

		# accounts that share home directories
		if ( exists $homedirs{$pw->dir} ) {
			$res = ask_question ('Should ' . $pw->name . ' and ' . $homedirs{$pw->dir} . ' share home directories?', ( exists $opts{'y'} ? 'yes': 'no'));
			push @sharedhome, ( $pw->name, $homedirs{$pw->dir} )
				if ( $res eq 'yes');
		}
		$homedirs{$pw->dir} = $pw->name;
	}
	$contents .= 'ignoreuid ' . join (' ', sort @uid0) . "\n"
		if ( scalar @uid0 );
	$contents .= 'ignorepass ' . join (' ', sort @nopass) . "\n"
		if ( scalar @nopass );
	$contents .= 'ignoremailfiles ' . join (' ', sort(unique(@sharedhome))) . "\n"
		if ( scalar @sharedhome );

	$contents .= 'mailfiles ' . join (' ', @{$config->{'mailfiles'}}) . "\n"
		if ( exists $config->{'mailfiles'} && scalar @{$config->{'mailfiles'}} );
	
	$contents .= 'intruderfiles ' . join (' ', @{$config->{'intruderfiles'}}) . "\n"
		if ( exists $config->{'intruderfiles'} && scalar @{$config->{'intruderfiles'}} );
	
	# Get the SMTP mail server
	my ($mailhost) = ask_question ('SMTP Mail server?', $opts{'m'});
	$contents .= "mailhost $mailhost\n";

	# Now setup the regex for the shared library files
	$contents .=  'sharedregex '
		. (( $^O eq 'darwin' )
			? 'lib[^/]*\.dylib$' 
			: 'lib[^/]*\.so(\..*)?$' )
		. "\n";

	# Now based on the OS, setup the ports root is allowed to login to.
	@array = ();

	# This will not work if we are running as non root, since on linux
	# the /etc/securetty file is read only by root.
	if ( ! $< && $^O eq 'linux' && -f '/etc/securetty' ) {
		local *FP;
		if (open (FP, '/etc/securetty' )) {
			while (<FP>) {
				chomp;
				push @array, $_;
			}
			close FP;
			$contents .= 'ignorerootports ' . join (' ', @array) . "\n"
				if ( scalar @array );
		} else {
			die "Could not open securetty\n";
		}
	} else {
		warn "Could not read securetty because we are not root\n"
			if ( $< && -f '/etc/securetty' );
		$contents .= "ignorerootports console\n";
	}

	# Any other 'special' system files that we should checksum by default
	@array = ();
	foreach my $f (@{$config->{'checksumfiles'}}) {
		push @array, $f
			if ( -e $f );
	}
	# Add our script, the config file and our modules to be checksummed.
	push @array, $opts{'P'} . '/sbin/security',
		$opts{'P'} . '/etc/security/security.cf',
		glob( $opts{'R'} . $opts{'M'} . '/Security/*.pm'),
		glob( $opts{'R'} . $opts{'M'} . '/Security/OS/*.pm'),
		glob( $opts{'R'} . $opts{'M'} . '/Security/util/*.pm');
	# Purge out the ROOTDIR value so it is not in the config file.
	my $i =0;
	map { $array[$i++] =~ s,$opts{'R'},,; } @array
		if ( length $opts{'R'} );
	$contents .= 'checksumfiles ' . join (' ', @array) . "\n"
		if ( scalar @array );

	# Now record the modules to load 
	# TODO: Need to verify that order is correct.
	my @load = glob( $opts{'R'} . $opts{'M'} . '/Security/*.pm');
	if ( scalar @load ) {
		use File::Basename;
		for ( my $i = 0; $i < scalar @load; $i++ ) {
			$load[$i] = basename $load[$i], '.pm';
		}
		$contents .= 'load ' . join (' ', @load) . "\n";
	} else {
		die "Could not find perl modules\n";
	}

	return $contents;
}

# Start of the main program
&getopts ($opts, \%opts);
usage() if ( $opts{'h'} );
$opts{'P'} = '' if ( ! exists $opts{'P'} );
$opts{'R'} = '' if ( ! exists $opts{'R'} );
$opts{'m'} = 'localhost' if ( ! exists $opts{'m'} );

# Now start generating the configuration files...
print &generate_config_file();

exit 0;
